/* klasė, vykdanti grid intervalinę paiešką
 */

#include "grid.h"
#include <iostream>

// CONSTRUCTORS

Grid::Grid()
{
	points = 0;
	cmp_count = 0;
	found_count = 0;
	setGridSize(1, 1);
}

Grid::Grid(const QRect &_area, int gridw, int gridh)
{
	points = 0;
	cmp_count = 0;
	found_count = 0;
	area = _area.normalize();
	setGridSize(gridw, gridh);
}


// DESTRUCTOR

Grid::~Grid()
{
	
}


// PRIVATE

// išvalo sąrašus
void Grid::clearLists()
{
	for (uint i=0; i<lists.count(); ++i)
	{
		for (uint j=0; j <lists[i].count(); ++j)
		{
			lists[i][j].clear();
		}
	}
}

// nustato sąrašų matricos dydį
void Grid::updateGrid()
{
	lists.resize((area.width()-1)/grid_width+1);
	for (uint i=0; i<lists.count(); ++i)
	{
		lists[i].resize((area.height()-1)/grid_height+1);
		for (uint j=0; j<lists[i].count(); ++j)
			lists[i][j].setAutoDelete(true);
	}
	listsChanged();
}

void Grid::updatePoint(QPoint *p)
{
	int x = (p->x() - area.left())/grid_width;
	int y = (p->y() - area.top())/grid_height;
	if (true) //((x>=0)&&(y>=0)&&((uint)x<lists.count())&&((uint)y<lists[x].count()))
	{
		QPoint *np = new QPoint();
		if (np!=0)
		{
			*np = *p;
			lists[x][y].append(np);
		}
		//std::cout << lists[x][y].count() << "\n";
	}
	else
	{
		std::cout << "updatePoint: x=" << x << ",y=" << y << 
			", lists[" << lists.count() << "][" << lists[0].count() << "]\n";
	}
}


// PUBLIC

void Grid::clear()
{
	if (points)
	{
		clearLists();
		points=0;
		cmp_count = 0;
		found_count = 0;
		pointsChanged();
		cmpChanged();
		foundChanged();
	}
}

// iš naujo taškus sukelia į sąrašus
void Grid::updatePoints()
{
	clearLists();
	if (points)
	{
		QPoint *p = points->first();
		while (p)
		{
			if (area.contains(*p))
			{
				updatePoint(p);
			}
			p = points->next();
		}
	}
	pointsChanged();
}

void Grid::setPoints(QPtrList<QPoint> *list)
{
	points = list;
	updatePoints();
}

void Grid::setGridSize(int width, int height)
{
	if ((width < 1)||(height < 1)) return;
	clearLists();
	grid_width = width;
	grid_height = height;
	updateGrid();
	updatePoints();
}

void Grid::setArea(const QRect &area)
{
	this->area = area.normalize();
	clearLists();
	updateGrid();
	updatePoints();
}

int Grid::getGridWidth()
{
	return grid_width;
}

int Grid::getGridHeight()
{
	return grid_height;
}

QRect &Grid::getArea()
{
	return area;
}

bool Grid::addPoint(QPoint *p)
{
	if ((!area.contains(*p))||(!points)) return false;
	points->append(p);
	updatePoint(p);
	pointsChanged();
	return true;
}

void Grid::removePoint(QPoint *p)
{
	if (!points) return;
	points->remove(p);
	if (!area.contains(*p)) return;
	int x = (p->x() - area.left())/grid_width;
	int y = (p->y() - area.top())/grid_height;
	lists[x][y].remove(p);
	pointsChanged();
}

QPtrList<QPoint> *Grid::search(const QRect &rect)
{
	QPtrList<QPoint> *list = new QPtrList<QPoint>();
	list->setAutoDelete(true);
	
	cmp_count = 0;
	found_count = 0;
	QRect nrect = area & rect.normalize();
	int startx = (nrect.left() - area.left())/grid_width;
	int endx = (nrect.right() - area.left())/grid_width;
	int starty = (nrect.top() - area.top())/grid_height;
	int endy = (nrect.bottom() - area.top())/grid_height;
	for (int i=startx; i<=endx; ++i)
	{
		for (int j=starty; j<=endy; ++j)
		{
			QPoint *p = lists[i][j].first();
			while (p)
			{
				if (rect.contains(*p))
				{
					QPoint *pp = new QPoint(*p);
					list->append(pp);
					++found_count;
				}
				++cmp_count;
				p = lists[i][j].next();
			}
		}
	}
	cmpChanged();
	foundChanged();
	
	return list;
}

int Grid::getListCount()
{
	return lists.count() * lists[0].count();
}

int Grid::getPointCount()
{
	if (points) return points->count();
	else return 0;
}

int Grid::getCmpCount()
{
	return cmp_count;
}

int Grid::getFoundCount()
{
	return found_count;
}

