/* grid metodą vizualizuojanti klasė GridView
 */

#include <qpainter.h>

#include "gridview.h"
#include "grid.h"

// point pictures
#include "./pav/point.xpm"
#include "./pav/fpoint.xpm"

// CONSTRUCTOR AND DESTRUCTOR

GridView::GridView(QWidget *parent, const char *name)
	:QFrame(parent, name)
{
	grid = new Grid(rect(), 50, 50);
	points = new QPtrList<QPoint>;
	points->setAutoDelete(true);
	grid->setPoints(points);
	found = 0;
	mode = gemReadOnly;
	point_img = new QPixmap((const char **)point);
	fpoint_img = new QPixmap((const char **)fpoint);
	setFrameStyle(WinPanel | Sunken);
}

GridView::~GridView()
{
	delete grid;
	delete points;
	if (found) delete found;
	delete point_img;
	delete fpoint_img;
}


// PROTECTED

void GridView::clearFound()
{
	if (found) 
	{
		delete found;
		found = 0;
	}
}

void GridView::drawPoint(QPainter *painter, const QPoint &p, bool f)
{
	if (f) painter->drawPixmap(QPoint(p.x()-2, p.y()-2), *fpoint_img);
	else painter->drawPixmap(QPoint(p.x()-2, p.y()-2), *point_img);
}

void GridView::paintEvent(QPaintEvent *e)
{
	QFrame::paintEvent(e);
	
	QPainter painter(this);
	painter.setPen(colorGroup().mid());
	
	int	w = grid->getArea().width() - frameWidth();
	int h = grid->getArea().height() - frameWidth();
	for (int i=0; i<w; i += grid->getGridWidth())
	{
		painter.drawLine(i, frameWidth(), i, h);
	}
	for (int j=0; j<h; j += grid->getGridHeight())
	{
		painter.drawLine(frameWidth(), j, w, j);		
	}
	if (selected)
	{
		painter.setPen(darkBlue);
		painter.drawRect(selection);
	}
	QPoint *p = points->first();
	while (p)
	{
		drawPoint(&painter, *p, false);
		p = points->next();
	}
	if (found)
	{
		QPoint *p = found->first();
		while (p)
		{
			drawPoint(&painter, *p, true);
			p = found->next();
		}
	}
}

void GridView::resizeEvent(QResizeEvent *e)
{
	QFrame::resizeEvent(e);
	grid->setArea(rect());
}


void GridView::mousePressEvent(QMouseEvent *e)
{
	QFrame::mousePressEvent(e);
	if (e->button() != Qt::LeftButton) return;
	switch (mode)
	{
		case gemNewPoints:
			addPoint(e->x(), e->y());
			break;
		case gemMovePoints:
			break;
		case gemSelectInterval: 
			start = e->pos();
			end = start;
			break;
	}
	if (mode != gemReadOnly) clearFound();
}

void GridView::mouseReleaseEvent(QMouseEvent *e)
{
	QFrame::mouseReleaseEvent(e);
	if (e->button() != Qt::LeftButton) return;
	switch (mode)
	{
		case gemMovePoints: break;
		case gemSelectInterval: 
			QRect r(start, end);
			r = r.normalize();
			if (selected) update(selection);
			update(r);
			selection = r;
			selected = true;
			update();
			break;
	}
	if (mode != gemReadOnly) clearFound();
}

void GridView::mouseMoveEvent(QMouseEvent *e)
{
	QFrame::mouseMoveEvent(e);
	if (e->state() != Qt::LeftButton) return;
	switch (mode)
	{
		case gemMovePoints: break;
		case gemSelectInterval:
			if (rect().contains(e->pos()))
			{
				//QPainter painter(this);
				//painter.setPen(black);
				//QRect r(start, end);
				//r.normalize();
				//painter.drawWinFocusRect(r);
				end = e->pos();
				//r = QRect(start, end);
				//r.normalize();
				//painter.drawWinFocusRect(r);
			}
			break;
	}
	if (mode != gemReadOnly) clearFound();
}


// PUBLIC

void GridView::addPoint(int x, int y)
{
	QPoint *p = new QPoint(x, y);
	grid->addPoint(p);
	QPainter painter(this);
	drawPoint(&painter, *p, false);
}

Grid *GridView::getGrid()
{
	return grid;
}

GridView::GridEditMode GridView::getMode()
{
	return mode;
}

void GridView::setMode(GridEditMode m)
{
	mode = m;
}

void GridView::setGridSize(int w, int h)
{
	grid->setGridSize(w, h);
	update();
}

void GridView::clear()
{
	grid->clear();
	points->clear();
	grid->setPoints(points);
	selected = false;
	clearFound();
	update();
}

void GridView::search()
{
	if (!selected) return;
	found = grid->search(selection);
	update();
}

