/* pagrindinio lango klasė MainWnd
 */
 
#include "mainwnd.h"
#include "gridview.h"
#include "grid.h"

#include <qobject.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qiconset.h>
#include <qpixmap.h>
#include <qbuttongroup.h>
#include <qstatusbar.h>
#include <qspinbox.h>
#include <qinputdialog.h>

#include <cstdlib>

MainWnd::MainWnd(QWidget *parent, const char *name)
	:QMainWindow(parent, name)
{
	QToolBar *toolbar = new QToolBar(this);
	
	gridview = new GridView(this);
	
	QButtonGroup *btn_grp = new QButtonGroup(this);
	btn_grp->setExclusive(true);
	btn_grp->hide();
	
	QToolButton *btn = new QToolButton(QIconSet(QPixmap("./pav/new.png")),
		"Valyti", QString::null, 0, 0, toolbar, 0);
	connect(btn, SIGNAL(clicked()), this, SLOT(clear()));
	btn->show();
	btn = new QToolButton(QIconSet(QPixmap("./pav/rpoints.png")),
		"Deti atsitiktinius taskus", QString::null, 0, 0, toolbar, 0);
	connect(btn, SIGNAL(clicked()), this, SLOT(putRandomPoints()));
	btn->show();
	btn = new QToolButton(QIconSet(QPixmap("./pav/points.png")),
		"Deti taskus", QString::null, 0, 0, toolbar, 0);
	btn->setToggleButton(true);
	btn_grp->insert(btn);
	connect(btn, SIGNAL(toggled(bool)), this, SLOT(toggledAddPoints(bool)));
	btn->show();
	btn = new QToolButton(QIconSet(QPixmap("./pav/select.png")),
		"Zymeti intervala", QString::null, 0, 0, toolbar, 0);
	btn->setToggleButton(true);
	btn_grp->insert(btn);
	connect(btn, SIGNAL(toggled(bool)), this,
		SLOT(toggledSelectInterval(bool)));
	btn->show();
	btn = new QToolButton(QIconSet(QPixmap("./pav/search.png")),
		"Ieskoti", QString::null, 0, 0, toolbar, 0);
	connect(btn, SIGNAL(clicked()), this, SLOT(search()));
	btn->show();
	
	toolbar->addSeparator();
	QLabel *l = new QLabel("Tinklelio dydis:", toolbar);
	l->show();
	QSpinBox *spin = new QSpinBox(toolbar);
	spin->setMinValue(0);
	spin->setValue(gridview->getGrid()->getGridWidth());
	connect(spin, SIGNAL(valueChanged(int)), this,
		SLOT(gridWidthChanged(int)));
	spin->show();
	spin = new QSpinBox(toolbar);
	spin->setMinValue(0);
	spin->setValue(gridview->getGrid()->getGridHeight());
	connect(spin, SIGNAL(valueChanged(int)), this,
		SLOT(gridHeightChanged(int)));
	spin->show();
	
	toolbar->show();
	
	QStatusBar *status = new QStatusBar(this);
	status->setSizeGripEnabled(true);
	
	setCentralWidget(gridview);
	
	point_count = new QLabel(status);
	status->addWidget(point_count, 1);
	list_count = new QLabel(status);
	status->addWidget(list_count, 1);
	cmp_count = new QLabel(status);  
	status->addWidget(cmp_count, 1);
	found_count = new QLabel(status);  
	status->addWidget(found_count, 1);
	updatePointCount();
	updateListCount();
	updateCmpCount();
	updateFoundCount();
	
	connect(gridview->getGrid(), SIGNAL(pointsChanged()),
		this, SLOT(updatePointCount(void)) );
	connect(gridview->getGrid(), SIGNAL(listsChanged()),
		this, SLOT(updateListCount(void)) );
	connect(gridview->getGrid(), SIGNAL(cmpChanged()),
		this, SLOT(updateCmpCount(void)) );
	connect(gridview->getGrid(), SIGNAL(foundChanged()),
		this, SLOT(updateFoundCount(void)) );
	
	srand(time(NULL));
}

MainWnd::~MainWnd()
{
	
}

void MainWnd::updatePointCount()
{
	point_count->setText(QString("Tasku skaicius: ").arg(
		gridview->getGrid()->getPointCount()));
}

void MainWnd::updateListCount()
{
	list_count->setText(QString("Sarasu skaicius: ").arg(
		gridview->getGrid()->getListCount()));
}

void MainWnd::updateCmpCount()
{
	cmp_count->setText(QString("Palyginimu skaicius: ").arg(
		gridview->getGrid()->getCmpCount()));
}

void MainWnd::updateFoundCount()
{
	found_count->setText(QString("Rasta tasku: ").arg(
		gridview->getGrid()->getFoundCount()));
}

void MainWnd::toggledAddPoints(bool on)
{
	if (on) gridview->setMode(GridView::gemNewPoints);
	else gridview->setMode(GridView::gemReadOnly);
}

void MainWnd::toggledSelectInterval(bool on)
{
	if (on) gridview->setMode(GridView::gemSelectInterval);
	else gridview->setMode(GridView::gemReadOnly);
}

void MainWnd::gridWidthChanged(int value)
{
	gridview->setGridSize(value, gridview->getGrid()->getGridHeight());
}

void MainWnd::gridHeightChanged(int value)
{
	gridview->setGridSize(gridview->getGrid()->getGridWidth(), value);
}

void MainWnd::clear()
{
	gridview->clear();
}

void MainWnd::search()
{
	gridview->search();
}

void MainWnd::putRandomPoints()
{
	bool ok;
	int count = QInputDialog::getInteger(QString("Deti atsitiktinius taskus"),
		QString("Tasku kiekis:"), 10, 0, 1000, 1, &ok);
	if (ok)
	{
		for (int i=0; i<count; ++i)
		{
			int x = rand()%gridview->width();
			int y = rand()%gridview->height();
			gridview->addPoint(x, y);
		}
	}
}

